#---------------------------------------------------------------------------
#
#   HIIW - Dialogues
#
#---------------------------------------------------------------------------

from textwrap import wrap
from pygame import Rect, K_RETURN, K_KP_ENTER
from albow.widget import Widget
from albow.controls import Label, Button
from albow.layout import Row, Column
from albow.dialogs import Dialog
from albow.palette_view import PaletteView
from albow.resource import get_image
from page_drawing import draw_sample

player_image = get_image("player.png")
page_image = get_image("page_written.png")

class IconView(Widget):

	def __init__(self, image):
		Widget.__init__(self, Rect(0, 0, 48, 48))
		self.image = image
	
	def draw(self, surf):
		frame = surf.get_rect()
		image = self.image
		r = image.get_rect()
		r.center = frame.center
		surf.blit(image, r)

class SampleViewBase(Widget):

	def __init__(self):
		Widget.__init__(self, Rect(0, 0, 350, 100))

	def draw(self, surf):
		sample = self.get_sample()
		if sample:
			draw_sample(surf, sample, surf.get_rect())

class SampleView(SampleViewBase):

	def __init__(self, sample):
		self.sample = sample
		SampleViewBase.__init__(self)
	
	def get_sample(self):
		return self.sample
	
class SelectedSampleView(SampleViewBase):

	def __init__(self, client):
		self.client = client
		SampleViewBase.__init__(self)

	def get_sample(self):
		page = self.client.game.state.selected_page
		if page and not page.destroyed:
			return page.sample

class CharacterPane(Row):

	def __init__(self, icon_image, contents_view):
		icon_view = IconView(icon_image)
		Row.__init__(self, [icon_view, contents_view], align = 't')

class SamplePane(CharacterPane):

	def __init__(self, icon_image, sample):
		sample_view = SampleView(sample)
		CharacterPane.__init__(self, icon_image, sample_view)

class TextPane(CharacterPane):

	def __init__(self, icon_image, text):
		lines = wrap(text, width = 50)
		text_view = Label("\n".join(lines))
		CharacterPane.__init__(self, icon_image, text_view)

class CharacterDialog(Dialog):

	def __init__(self, panes, ok_text = "OK", cancel_text = None):
		Dialog.__init__(self)
		panes_view = Column(panes, align = 'l')
		buttons = [Button(ok_text, action = self.ok)]
		if cancel_text:
			buttons.append(Button(cancel_text, action = self.cancel))
		buttons_view = Row(buttons)
		contents = Column([panes_view, buttons_view], align = 'r')
		contents.topleft = (20, 20)
		self.add(contents)
		self.shrink_wrap()

class PageSelectorView(PaletteView):

	def __init__(self, client):
		self.client = client
		PaletteView.__init__(self, (35, 45), 3, 9, True)
	
	def num_items(self):
		return len(self.client.pages)
	
	def item_is_selected(self, i):
		return self.client.game.state.selected_page is self.client.pages[i]
	
	def click_item(self, i, e):
		self.client.game.state.select_page(self.client.pages[i])
	
	def draw_item(self, surf, i, rect):
		image = page_image
		r = image.get_rect()
		r.center = rect.center
		surf.blit(image, r)

class BattleDialog(Dialog):

	def __init__(self, game, enemy, pages):
		Dialog.__init__(self)
		self.game = game
		self.enemy = enemy
		self.pages = pages
		if not game.state.selected_page and self.pages:
			self.game.state.select_page(self.pages[0])
		enemy_pane = SamplePane(enemy.image, enemy.challenge)
		sample_pane = CharacterPane(player_image, SelectedSampleView(self))
		pages_view = PageSelectorView(self)
		buttons = [
			Button("Wield These Words", action = self.ok),
			Button("Cancel", action = self.cancel)]
		panes_view = Column([enemy_pane, sample_pane, pages_view],
			align = 'l')
		buttons_view = Row(buttons)
		contents = Column([panes_view, buttons_view],
			align = 'r')
		contents.topleft = (20, 20)
		self.add(contents)
		self.shrink_wrap()
	
	def ok(self):
		page = self.game.state.selected_page
		#print "BattleDialog: Attacking with", page ###
		if page and not page.destroyed:
			if page is self.enemy.response_page:
				self.dismiss(True)
			else:
				player_alert("Methinks those lines do not belong togeth'r.")

	def key_down(self, e):
		if e.key == K_RETURN or e.key == K_KP_ENTER:
			self.ok()
		else:
			Dialog.key_down(self, e)

def do_pre_battle_dialogue(enemy):
	enemy_pane = SamplePane(enemy.image, enemy.challenge)
	player_pane = TextPane(player_image,
		"Forsooth, a hallucination! And with the gall to quote mine own "
		"words at me. I assuredly penned a fitting rejoinder to that line, if "
		"I can but find the page...")
	dlog = CharacterDialog([enemy_pane, player_pane], "Do Battle", "Cancel")
	return dlog.present()

def do_battle_dialogue(enemy):
	from game import game
	pages = [page for page in game.state.items['page']
		if not page.destroyed]
	if pages:
		dlog = BattleDialog(game, enemy, pages)
		return dlog.present()
	else:
		name = enemy.name.title()
		player_alert("I needs must come back and deal with %s when I have "
			"found some of my lost pages." % name)

def do_enemy_dialogue(enemy):
	if do_pre_battle_dialogue(enemy):
		return do_battle_dialogue(enemy)

def player_alert(text):
	player_pane = TextPane(player_image, text)
	dlog = CharacterDialog([player_pane])
	dlog.present()
