#---------------------------------------------------------------------------
#
#   HIIW - Folio management view
#
#---------------------------------------------------------------------------

from pygame import Rect
from albow.widget import Widget
from albow.resource import get_image
from albow.image_array import get_image_array
from game import game
from page_drawing import draw_page
from help import HelpBox

#  Corner button states
SLIDE_DOWN = 0
SLIDE_UP = 1
INSERT = 2

#  Swap button states
SWAP = 0
REMOVE = 1

def calc_rewrite_button_rect(ptl, pi, bi):
	rp = pi.get_rect()
	rp.topleft = ptl
	rb = bi.get_rect()
	rb.bottomright = rp.bottomright
	return rb

class FolioView(Widget):

	folio_bg_image = get_image("folio_open.png")
	paper_image = get_image("paper.png")
	corner_button_images = get_image_array("page_corner_buttons.png", 3, border = 2)
	swap_button_images = get_image_array("page_swap_button.png", 2, border = 2)
	rewrite_button_image = get_image("rewrite_button.png")
	
	paper_width = paper_image.get_width()
	paper_height = paper_image.get_height()
	corner_button_rect = corner_button_images[0].get_rect()
	
	current_page_topleft = (250, 30)
	swap_button_topleft = (210, 170)
	rewrite_button_rect = calc_rewrite_button_rect(
		current_page_topleft, paper_image, rewrite_button_image)

	#  Following coordinates are relative to bottom left, y upwards
	folio_pages_topleft = (20, 100)
	folio_bg_offset = (-230, 20)
	page_h_spacing = corner_button_rect.width
	selected_page_v_offset = paper_height * 2 // 3
	page_v_stagger = 2
	
	def __init__(self, *args, **kwds):
		Widget.__init__(self, *args, **kwds)
		self.add(HelpBox("page_view.hlp"))
		self.add(HelpBox("folio_view.hlp"))
		self.add(HelpBox("folio_orange_buttons.hlp"))
		self.add(HelpBox("folio_rewrite.hlp", rewrite_button_visible))
	
	def get_visible(self):
		return game.folio_visible

	def draw(self, surf):
		frame = surf.get_rect()
		folio = game.state.selected_folio
		current_page = game.state.selected_page
		if current_page:
			draw_page(surf, current_page, self.current_page_topleft)
			if current_page.destroyed:
				img = self.rewrite_button_image
				surf.blit(img, self.rewrite_button_rect)
		if folio:
			sel = folio.selected_page
			bg = self.folio_bg_image
			w = bg.get_width()
			x0, y0 = frame.bottomleft
			px0, py0 = self.folio_pages_topleft
			bdx, bdy = self.folio_bg_offset
			surf.blit(bg, (x0 + px0 + bdx, y0 - py0 - bdy))
			paper_image = self.paper_image
			for info in self.iter_page_info(folio):
				page = info.page
				if current_page:
					s = INSERT
				else:
					s = None
				if page:
					draw_page(surf, page, info.page_topleft)
					if not page.is_dummy:
						if page is sel:
							s = SLIDE_DOWN
						else:
							s = SLIDE_UP
				if s is not None:
					corner_rect = info.corner_rect
					if corner_rect:
						img = self.corner_button_images[s]
						r = img.get_rect()
						r.center = corner_rect.center
						surf.blit(img, r)
				if sel:
					if current_page:
						s = SWAP
					else:
						s = REMOVE
					surf.blit(self.swap_button_images[s], self.swap_button_topleft)

	def mouse_down(self, e):
		p = e.local
		folio = game.state.selected_folio
		if folio:
			for info in self.iter_page_info(folio):
				corner = info.corner_rect
				if corner and corner.collidepoint(p):
					self.click_page_corner(folio, info)
					return
			if folio.selected_page:
				r = self.swap_button_rect()
				if r.collidepoint(p):
					self.click_swap_button(folio)
		page = game.state.selected_page
		if page and page.destroyed:
			r = self.rewrite_button_rect
			if r.collidepoint(p):
				self.click_rewrite_button()
	
	def click_page_corner(self, folio, info):
		page = info.page
		if page:
			if page is folio.selected_page:
				folio.selected_page = None
			else:
				folio.selected_page = page
		else:
			page = game.state.selected_page
			if page:
				game.state.remove_item_from_inventory(page)
				folio.pages[info.page_no] = page
	
	def click_swap_button(self, folio):
		state = game.state
		n = folio.selected_page_no
		fpage = folio.pages[n]
		ipage = state.selected_page
		state.remove_item_from_inventory(ipage)
		folio.pages[n] = ipage
		state.add_item_to_inventory(fpage)
		state.selected_page = fpage
	
	def click_rewrite_button(self):
		game.rewrite_page()
	
	def swap_button_rect(self):
		r = self.swap_button_images.get_rect()
		r.topleft = self.swap_button_topleft
		return r
	
	def iter_page_info(self, folio):
		x0 = 0
		y0 = self.height
		px0, py0 = self.folio_pages_topleft
		dx = self.page_h_spacing
		paper_width = self.paper_width
		pages = folio.pages
		sel = folio.selected_page
		y = y0 - py0
		for i in xrange(len(pages) - 1, -1, -1):
			x = x0 + px0 + i * dx
			info = FolioPageInfo()
			info.page_no = i
			page = pages[i]
			info.page = page
			yp = y
			if page:
				if page is sel:
					yp -= self.selected_page_v_offset
				info.page_topleft = (x, yp)
			if not page or not page.is_dummy:
				rect = Rect(self.corner_button_rect)
				rect.topright = (x + paper_width, yp)
				info.corner_rect = rect
			yield info
			y += self.page_v_stagger

#---------------------------------------------------------------------------

class FolioPageInfo(object):
	page = None
	page_no = None
	page_topleft = None
	corner_rect = None

def rewrite_button_visible():
	page = game.state.selected_page
	return page and page.destroyed
