#---------------------------------------------------------------------------
#
#   HIIW - Game
#
#---------------------------------------------------------------------------

import os, gc
from copy import deepcopy
from tempfile import mkstemp
from cPickle import Pickler, Unpickler
from albow.resource import resource_path, get_sound
from albow.dialogs import ask, alert
from albow.file_dialogs import look_for_file_or_directory
from house import House, NORTH, SOUTH, EAST, WEST
from items import item_kinds
from options import options
from dialogues import player_alert

#---------------------------------------------------------------------------

writing_sound = get_sound("writing.ogg")

#---------------------------------------------------------------------------

class State(object):
	#  house             House
	#  is_completed      boolean
	#  player_location   (cx, cy)
	#  current_floor     Floor
	#  items             {kind: [Item]}
	#  selected_item     Item
	#  selected_folio    Folio
	#  selected_page     Page
	
	is_completed = False
	selected_item = None
	selected_folio = None
	selected_page = None

	def __init__(self):
		self.house = House(options.seed_option)
		self.current_floor = self.house.floors[0]
		self.items = {}
		for kind in item_kinds:
			self.items[kind] = []
		for item in self.house.leftover_items:
			self.put_item_in_inventory_slot(item)
		self.leftover_items = None
		self.player_location = self.house.get_starting_location()
	
	def select_page(self, page):
		self.selected_page = page
	
	def iter_inventory_items(self):
		for items in self.items.itervalues():
			for item in items:
				yield item
	
	def put_item_in_inventory_slot(self, item):
		if item:
			slot = self.items[item.kind]
			if item not in slot:
				slot.append(item)
				if item.kind == 'folio' and not self.selected_folio:
					self.selected_folio = item
				if item.kind == 'page' and not self.selected_page:
					self.selected_page = item
				return True
		return False

	def add_item_to_inventory(self, item):
		if self.put_item_in_inventory_slot(item):
			game.unsaved_progress = True
	
	def remove_item_from_inventory(self, item):
		if item:
			slot = self.items[item.kind]
			if item in slot:
				self.items[item.kind].remove(item)
				if item is self.selected_item:
					self.selected_item = None
				if item is self.selected_page:
					self.selected_page = None
				game.unsaved_progress = True
	
	def check_whether_completed(self):
		return self.house.check_whether_completed()
	
#---------------------------------------------------------------------------

class HIIWGame(object):
	#  save_dir             string     Last save file dir
	#  save_name            string     Last save file name
	#  unsaved_progress     boolean    Game progress needs saving
	#  help_on              boolean
	#  folio_visible        boolean
		
	app_version = (1, 0, 0)  #  Version number of the game application
	app_magic = "HIIW"

	#  Saved game file constants

	save_file_version = 1         #  Version number of data we write
	save_min_version = 1          #  Minimum data version we understand
	save_app_version = (1, 0, 0)  #  Earliest app version that understands what we write
	save_version_ranges = {}      #  For earlier data versions, version -> (min_app_version, max_app_version)
	save_file_magic = "SAVE"      #  Magic for recognising type of data
	
	#  Filenames and  suffixes
	
	save_file_default_name = "SavedGame"
	save_file_suffix = ".hiiwg"   #  Saved game files
	
	# The following paths are relative to the directory containing
	# the Game directory or the .app bundle.
	
	default_save_dir_name = "Saves"

	#  Optional feature flags
	
	disable_gc_during_play = False

	#  Default values of instance variables

	state = None
	save_dir = None
	save_name = None
	unsaved_progress = False
	help_on = False
	folio_visible = False

	def create_level(self):
		return Level()

	def game_in_progress(self):
		return self.state and not self.state.is_completed
	
	def game_needs_saving(self):
		return self.game_in_progress() and self.unsaved_progress

	def new_game(self):
		self.state = State()
		self.unsaved_progress = False
	
	def load_game(self, path):
		f = open(path, "rb")
		check_file_magic(f, self.app_magic)
		check_file_magic(f, self.save_file_magic)
		p = Unpickler(f)
		file_version = p.load()
		app_version = p.load()
		check_file_version(file_version, app_version,
			self.save_min_version, self.save_file_version,
			self.save_version_ranges)
		state = p.load()
		f.close()
		self.state = state
		self.set_save_path(path)
		self.unsaved_progress = False

	def save_game(self, path):
		fd, tmp_path = mkstemp(dir = os.path.dirname(path))
		f = os.fdopen(fd, "wb")
		f.write(self.app_magic)
		f.write(self.save_file_magic)
		p = Pickler(f, 2)
		p.dump(self.save_file_version)
		p.dump(self.save_app_version)
		p.dump(self.state)
		f.close()
		try:
			os.unlink(path)
		except EnvironmentError:
			pass
		os.rename(tmp_path, path)
		self.set_save_path(path)
		self.unsaved_progress = False

	def get_save_dir(self):
		"""Returns the full pathname of the directory last used to save a
		game, or the default save directory."""
		return self.save_dir or self.get_default_dir(self.default_save_dir_name)
	
	def get_save_path(self):
		return os.path.join(self.get_save_dir(),
			self.save_name or self.save_file_default_name)
	
	def set_save_path(self, path):
		self.save_dir, self.save_name = os.path.split(path)
	
	def get_default_dir(self, name):
		from albow.resource import resource_dir
		d = os.path.dirname
		base = d(d(resource_dir))
		return os.path.join(base, name)
	
	def begin_frame(self):
		pass
	
	def check_whether_completed(self):
		state = self.state
		if state.check_whether_completed():
			state.is_completed = True
			return True
		else:
			return False

	def get_current_floor(self):
		return self.state.current_floor
	
	current_floor = property(get_current_floor)

	def move_player(self, dx, dy):
		state = self.state
		floor = self.current_floor
		cx1, cy1 = state.player_location
		cx2 = cx1 + dx
		cy2 = cy1 + dy
		dir = delta_to_direction(dx, dy)
		cell1 = floor.cells[cx1][cy1]
		if cell1.upstairs_dir == dir:
			floor = cell1.room.upstairs_room.floor
		elif cell1.downstairs_dir == dir:
			floor = cell1.room.downstairs_room.floor
		else:
			wall = floor.find_wall_between((cx1, cy1), (cx2, cy2))
			if wall:
				if wall.is_wall:
					return
				obstacle =  wall.obstacle
				if obstacle:
					if obstacle.do_dialogue():
						wall.obstacle = None
						obstacle.report_removal()
					return
		cell2 = floor.cells[cx2][cy2]
		furn = cell2.furniture
		if furn and furn.is_impassable:
			return
		state.current_floor = floor
		state.player_location = (cx2, cy2)
		self.unsaved_progress = True
		item = cell2.item
		if item:
			item.picked_up()
			state.add_item_to_inventory(item)
			cell2.item = None
	
	def rewrite_page(self):
		cell = self.player_cell()
		if not cell.can_write_here:
			player_alert("I find inspiration strikes best when I am seated at my "
				"writing desk.")
			return
		need = []
		if self.inventory_count('paper') == 0:
			need.append("blank paper")
		if self.inventory_count('quill') == 0:
			need.append("quills")
		if self.inventory_count('ink') == 0:
			need.append("ink")
		if need:
			if len(need) >= 3:
				need[:-1] = ",".join(need[:-1])
			s = " and ".join(need)
			player_alert("Alas, I cannot rewrite this page until I find some more %s!"
				% s)
			return
		page = self.state.selected_page
		page.destroyed = False
		self.use_up('paper')
		self.use_up('quill')
		self.use_up('ink')
		writing_sound.play()
	
	def player_cell(self):
		state = self.state
		x, y = state.player_location
		return state.current_floor.cells[x][y]
	
	def inventory_count(self, kind):
		items = self.state.items.get(kind)
		if items:
			return len(items)
		else:
			return 0
	
	def use_up(self, kind):
		items = self.state.items.get(kind)
		if items:
			item = items[0]
			if item.use_up():
				del items[0]

	def toggle_help(self):
		self.help_on = not self.help_on
	
	def toggle_folio(self):
		self.folio_visible = not self.folio_visible

def delta_to_direction(dx, dy):
	if dx > 0:
		return EAST
	elif dx < 0:
		return WEST
	elif dy > 0:
		return SOUTH
	elif dy < 0:
		return NORTH

#---------------------------------------------------------------------------

def check_file_magic(f, expected):
	magic = f.read(len(expected))
	if magic <> expected:
		raise EnvironmentError("Does not seem to be a saved game file.")

def check_file_version(file_version, app_version,
		min_version, cur_version, version_ranges):
	if not min_version <= file_version <= cur_version:
		if file_version > cur_version:
			mess = "Can only be opened by version %s.%s.%s or later." % app_version
		else:
			v1, v2 = version_ranges[file_version]
			vs1 = "%s.%s.%s" % v1
			vs2 = "%s.%s.%s" % v2
			mess = "Can only be opened by versions %s to %s." % (vs1, vs2)
		raise EnvironmentError(mess)

#---------------------------------------------------------------------------

game = HIIWGame()
