#---------------------------------------------------------------------------
#
#   HIIW - House
#
#---------------------------------------------------------------------------

from __future__ import division
from collections import defaultdict
from directions import *
from random_mapping import RandomMapping
from items import Quill, Ink, Folio
from plays import get_play_samples
from barnacles import barnacle
from room_styles import create_floor_styles
from obstacles import generate_enemy
from furniture import WritingDesk

debug_place = False
debug_place_remaining_items = False

floor_size = (12, 12)
#num_floors = 1
min_room_size = 3
max_room_size = 12
min_room_area = 9
max_room_area = 81
room_subdivision_chance = 0.5
num_plays = 5
samples_per_play = 5

#---------------------------------------------------------------------------

class ItemPlacementState(list):
	#  rooms_remaining   int
	
	def __init__(self):
		self.pending_units = defaultdict(int)

	def items_per_room(self):
		n = len(self)
		r = self.rooms_remaining
		return (n + r - 1) // r
	
	def add_unit(self, item_class):
		pending = self.pending_units
		pending[item_class] += 1
		if pending[item_class] == item_class.initial_units:
			self.append(item_class())
			pending[item_class] = 0
	
	def flush_pending(self):
		for item_class, units in self.pending_units.iteritems():
			if units > 0:
				self.append(item_class())

#---------------------------------------------------------------------------

class Cell(object):
	#  room            Room
	#  item            Item
	#  furniture       Furniture
	#  keep_clear      boolean
	#  can_write_here  boolean
	
	item = None
	furniture = None
	upstairs_dir = None
	downstairs_dir = None
	keep_clear = False
	can_write_here = False

#---------------------------------------------------------------------------

class Wall(object):
	#  is_wall       boolean
	#  is_exterior   boolean
	#  rooms         Room[2]   Adjacent rooms
	#  obstacle      Enemy or Door
	
	is_wall = False
	is_exterior = False
	obstacle = None
	
	def __init__(self):
		self.rooms = [None, None]
	
	def other_room(self, room):
		rooms = self.rooms
		if room is rooms[0]:
			return rooms[1]
		elif room is rooms[1]:
			return rooms[0]

#---------------------------------------------------------------------------

class Room(object):
	#  rnd           RandomMapping
	#  floor         Floor
	#  bounds        (int, int, int, int)
	#  exterior      (bool, bool, bool, bool)
	#  doorways      [Wall]
	#  carpet_no     index into list of Carpets
	#  available_item_locations
	#                iterator over boundary cells in random order
	#  available_furniture_locations
	#                list of (x, y, dir) in random order
	
	rnd = barnacle('rnd')
	available_item_locations = barnacle('available_item_locations')
	available_furniture_locations = barnacle('available_furniture_locations')
	
	carpet_no = None
	upstairs_room = None
	downstairs_room = None

	def __init__(self, rnd, floor, floor_style, bounds, exterior):
		self.rnd = rnd
		self.floor = floor
		self.bounds = bounds
		self.exterior = exterior
		self.doorways = []
		locations = list(self.iter_boundary_cells())
		self.available_item_locations = rnd.permute("item_locations", locations)
		self.available_furniture_locations = list(rnd.permute("furniture_locations",
			list(self.iter_furniture_locations())))
		room_style = floor_style.get_room_style(rnd)
		room_style.decorate_room(self)
	
	def __str__(self):
		return "Room<F%s,%s>" % (self.floor.no, self.bounds)
	
	def cell(self, loc):
		x, y = loc
		return self.floor.cells[x][y]

	def connect_to_adjacent_rooms(self):
		n = self.num_boundary_walls()
		for i in self.rnd.permuted_range("walls", n):
			wall = self.wall_with_index(i)
			other = wall.other_room(self)
			if other and not self.is_connected_to(other):
				wall.is_wall = False
				self.doorways.append(wall)
				other.doorways.append(wall)
				return
	
	def num_boundary_walls(self):
		#  Number of walls suitable for doorways.
		#  Omits corner walls.
		l, t, r, b = self.bounds
		return 2 * (r - l - 2 + b - t - 2)
	
	def wall_with_index(self, i):
		#  i'th wall suitable for doorway.
		#  Omits corner walls.
		l, t, r, b = self.bounds
		w = r - l - 2
		j, k = divmod(i, 2)
		if j >= w:
			d = VERT
			x = l + k
			y = t + 1 + j - w
		else:
			d = HORZ
			x = l + 1 + j
			y = t + k
		return self.floor.walls[d][x][y]
	
	def is_connected_to(self, other):
		for wall in self.doorways:
			if other in wall.rooms:
				return True
		return False
	
	def place_items(self, items, visited_rooms):
		if debug_place:
			print "Room.place_items:", self
		rnd = self.rnd
		visited_rooms.add(self)
		subrooms = [room for room in self.iter_connected_rooms()
			if room not in visited_rooms]
		for subroom in rnd.permute("subplace", subrooms):
			subroom.place_items(items, visited_rooms)
			# Place obstacle in doorway and add key items here
			enemy = generate_enemy(self.rnd, items)
			if enemy:
				if debug_place:
					print "Placing enemy", enemy.name, "in", self, "response", \
						enemy.response_page
				wall = self.doorway_to_room(subroom)
				if wall:
					wall.obstacle = enemy
		nitems = min(items.items_per_room(), len(items))
		if debug_place:
			print "Placing", nitems, "items in", self
		chosen_items = rnd.choose_and_pop("choose_items", nitems, items)
		for (cx, cy), item in zip(self.available_item_locations, chosen_items):
			if debug_place:
				print "Placing item", item, "at", (cx, cy)
			cell = self.floor.cells[cx][cy]
			assert not cell.item
			cell.item = item
			nitems -= 1
			item.add_prerequisite_items(items)
		items.rooms_remaining -= 1
		if debug_place_remaining_items:
			print "Remaining items:", items

	def doorway_to_room(self, room):
		for wall in self.doorways:
			if room in wall.rooms:
				return wall
		print "*** Doorway not found from", self, "to", room
	
	def place_furniture(self, key, furn):
		rnd = self.rnd
		cells = self.floor.cells
		locations = self.available_furniture_locations
		for i, (x, y, dirs) in enumerate(locations):
			cell = cells[x][y]
			if not cell.furniture:
				locations.pop(i)
				dir = rnd.choice(key, dirs)
				if debug_place:
					print "Placing furniture at", (x, y), "facing", dir, "in", self
				furn.place_in_room(self, (x, y), dir)
				return

	def iter_boundary_cells(self):
		l, t, r, b = self.bounds
		for x in xrange(l, r):
			yield (x, t)
			yield (x, b - 1)
		for y in xrange(t + 1, b - 1):
			yield	(l, y)
			yield (r - 1, y)
	
	def iter_furniture_locations(self):
		l, t, r, b = self.bounds
		cells = self.floor.cells
		for x, y in self.iter_boundary_cells():
			if not cells[x][y].keep_clear:
				dirs = []
				if x < r - 1:
					dirs.append(EAST)
				if x > l:
					dirs.append(WEST)
				if y < b - 1:
					dirs.append(SOUTH)
				if y > t:
					dirs.append(NORTH)
				yield (x, y, dirs)
	
	def iter_connected_rooms(self):
		for wall in self.doorways:
			yield wall.other_room(self)
		if self.downstairs_room:
			yield self.downstairs_room
		if self.upstairs_room:
			yield self.upstairs_room

	def dump(self):
		print self.bounds, self.exterior

#---------------------------------------------------------------------------

class Floor(object):
	#  rnd     RandomMapping
	#  no      int
	#  cells   Cell[x][y]
	#  walls   Wall[hv][x][y]
	#  rooms   [Room]

	rnd = barnacle('rnd')

	def __init__(self, rnd, no):
		def walls(xdim, ydim):
			return [[
				Wall() for y in xrange(ydim)]
					for x in xrange(xdim)]
		self.rnd = rnd
		self.no = no
		fw, fh = floor_size
		self.rooms = []
		self.cells = [[
			Cell() for x in xrange(fw)]
				for y in xrange(fh)]
		self.walls = [
			walls(fw, fh + 1),
			walls(fw + 1, fh)]
	
	def generate_rooms(self, floor_style):
		fw, fh = floor_size
		dir = self.rnd.range("div_dir", 2)
		self.generate_rooms_for_zone(self.rnd, dir, floor_style,
			(0, 0, fw, fh),
			(True, True, True, True))
	
	def generate_rooms_for_zone(self, prnd, dir, floor_style, bounds, exterior):
		rnd = prnd.fork(bounds)
		if should_subdivide_room(rnd, dir, bounds):
			sub_bounds, sub_ext = subdivide_room(rnd, dir, bounds, exterior)
			sub_dir = 1 - dir
			self.generate_rooms_for_zone(rnd, sub_dir, floor_style, sub_bounds[0], sub_ext[0])
			self.generate_rooms_for_zone(rnd, sub_dir, floor_style, sub_bounds[1], sub_ext[1])
		else:
			self.generate_room(rnd, floor_style, bounds, exterior)
	
	def generate_room(self, rnd, floor_style, bounds, exterior):
		#print "Floor.generate_room:", bounds ###
		room = Room(rnd, self, floor_style, bounds, exterior)
		self.rooms.append(room)
		l, t, r, b = bounds
		el, et, er, eb = exterior
		hrange = xrange(l, r)
		vrange = xrange(t, b)
		cells = self.cells
		hwalls = self.walls[HORZ]
		vwalls = self.walls[VERT]
		for x in hrange:
			for y in vrange:
				cells[x][y].room = room
		def put_wall(wall, ext, side):
			wall.is_wall = True
			wall.is_exterior = ext
			wall.rooms[side] = room
		for x in hrange:
			put_wall(hwalls[x][t], et, 1)
			put_wall(hwalls[x][b], eb, 0)
		for y in vrange:
			put_wall(vwalls[l][y], el, 1)
			put_wall(vwalls[r][y], er, 0)

	def connect_rooms(self):
		for room in self.rooms:
			room.connect_to_adjacent_rooms()
	
	def find_wall_between(self, coords1, coords2):
		cx1, cy1 = coords1
		cx2, cy2 = coords2
		d = cy1 == cy2
		cx = max(cx1, cx2)
		cy = max(cy1, cy2)
		return self.walls[d][cx][cy]

	def dump(self):
		for i, room in self.rooms:
			print "Room", i
			room.dump()

	def put_walls_around_stairs(self, x, y, d):
		walls = self.walls
		walls[HORZ][x][y].is_wall = d != SOUTH
		walls[HORZ][x][y+1].is_wall = d != NORTH
		walls[VERT][x][y].is_wall = d != EAST
		walls[VERT][x+1][y].is_wall = d != WEST

def should_subdivide_room(rnd, dir, bounds):
	l, t, r, b = bounds
	w = r - l
	h = b - t
	a = w * h
	if a <= min_room_area:
		return False
	if a > max_room_area:
		return True
	if dir == VERT:
		if w <= 2 * min_room_size:
			return False
		elif w > max_room_size:
			return True
	else:
		if h <= 2 * min_room_size:
			return False
		elif h > max_room_size:
			return True
	return rnd.chance("div_chance", room_subdivision_chance)

def subdivide_room(rnd, dir, bounds, exterior):
	l, t, r, b = bounds
	el, et, er, eb = exterior
	if dir == HORZ:
		p = rnd.range("div_pos", t + min_room_size, b - min_room_size)
		b0 = (l, t, r, p)
		b1 = (l, p, r, b)
		e0 = (el, et, er, False)
		e1 = (el, False, er, eb)
	else:
		p = rnd.range("div_pos", l + min_room_size, r - min_room_size)
		b0 = (l, t, p, b)
		b1 = (p, t, r, b)
		e0 = (el, et, False, eb)
		e1 = (False, et, er, eb)
	return (b0, b1), (e0, e1) 

#---------------------------------------------------------------------------

class House(object):
	#  floors          [Floor]
	#  folios          [Folio]
	#  leftover_items  [Item]

	def __init__(self, seed = None):
		self.floors = []
		self.generate_house(seed)
	
	def generate_house(self, seed):
		rnd = RandomMapping(seed = seed)
		floor_styles = create_floor_styles()
		#for i in xrange(num_floors):
		for i, floor_style in enumerate(floor_styles):
			floor = Floor(rnd.fork(("floor", i)), i)
			floor.generate_rooms(floor_style)
			floor.connect_rooms()
			self.floors.append(floor)
		self.generate_stairs()
		fw, fh = floor_size
		sx = rnd.range("startx", fw)
		sy = rnd.range("starty", fh)
		self.starting_location = (sx, sy)
		start_room = self.floors[0].cells[sx][sy].room
		desk = WritingDesk()
		start_room.place_furniture("writing_desk", desk)
		self.generate_and_place_items(rnd, start_room)
	
	def generate_stairs(self):
		floors = self.floors
		for i in xrange(len(floors) - 1):
			self.generate_stairs_between(floors[i], floors[i+1])
	
	def generate_stairs_between(self, lower_floor, upper_floor):
		#print "Floor.generate_stairs_between:", lower_floor, upper_floor ###

		def stair_pos(fs):
			return rnd.range("stairs_x", 2, fs - 1)

		def pick_stair_loc(s):
			if d == WEST:
				lx = stair_pos(fw)
				if s == 0:
					ly = 0
				else:
					ly = fh - 1
			else:
				ly = stair_pos(fh)
				if s == 0:
					lx = 0
				else:
					lx = fw - 1
			return (lx, ly)

		fw, fh = floor_size
		rnd = lower_floor.rnd
		d = rnd.range("stairs_dir", 2, 4)
		s = rnd.range("stairs_side", 2)
		lx, ly = pick_stair_loc(s)
		#  If conflict with existing stairway, switch to other side
		if lower_floor.cells[lx][ly].keep_clear:
			pick_stair_loc(s ^ 2)
		ux = lx
		uy = ly
		if d & 1 == 0:
			ux = lx - 1
			uxc = ux - 1
			uyc = uy
		else:
			uy = ly - 1
			uyc = uy - 1
			uxc = ux
		lower_cell = lower_floor.cells[lx][ly]
		upper_cell = upper_floor.cells[ux][uy]
		lower_cell.upstairs_dir = d
		upper_cell.downstairs_dir = d ^ 2
		lower_cell.room.upstairs_room = upper_cell.room
		upper_cell.room.downstairs_room = lower_cell.room
		#  Keep space in front of upper stairway clear
		upper_floor.cells[uxc][uyc].keep_clear = True
		lower_floor.put_walls_around_stairs(lx, ly, d)
		upper_floor.put_walls_around_stairs(ux, uy, d ^ 2)
	
	def generate_and_place_items(self, rnd, start_room):
		items = ItemPlacementState()
		items.rooms_remaining = self.room_count()
		folios = []
		for play in get_play_samples(rnd, num_plays, samples_per_play):
			frnd = rnd.fork(play.name)
			folio = Folio(frnd, play)
			folios.append(folio)
			items.append(folio)
			items.extend(folio.missing_pages)
		self.folios = folios
		#num_items = len(items)
		#items_per_room = (num_items + num_rooms - 1) // num_rooms
		visited_rooms = set()
		if debug_place:
			print "Items to place:", items
		start_room.place_items(items, visited_rooms)
		items.flush_pending()
		if debug_place:
			print "Leftover items:", items
		self.leftover_items = items

	def get_starting_location(self):
		return self.starting_location
	
	def room_count(self):
		return sum(len(floor.rooms) for floor in self.floors)
	
	def check_whether_completed(self):
		for folio in self.folios:
			if not folio.check_whether_completed():
				return False
		return True

	def dump(self):
		for i, floor in enumerate(self, rooms):
			print "Floor", i
			floor.dump()
