##################################################################
#
#   Humerus - Game object
#
#   The central object of a game made up of a series of levels.
#   Provides facilities for remembering which levels have been
#   completed and saving this information in a saved-game file.
#   Designed to be used in conjunction with an HShell.
#
#   Levels are kept in a directory, one level per file. The
#   order of playing is determined by lexicographically sorting
#   the filenames. It is recommended to prefix each filename by
#   a level number with leading zeroes.
#
#   By default, levels are taken from a directory called
#   "levels.xxx" in the Resources directory, where "xxx" is
#   your chosen level set suffix. Support is provided for
#   letting the user choose another set of levels to play.
#
#   If any information besides the list of completed levels needs
#   to be stored in the saved game file, it should be kept as
#   pickleable attributes of the game's 'state' object.
#
#   Abstract methods:
#
#      read_level(self, path)
#         Should read a level from the specified file and
#         return an object representing it.
#
#      new_state(self)
#         Should create and return a new state object appropriately
#         for a new game. May be left unimplemented if no additional
#         game state needs to be saved.
#
#   A level can be represented by any object having the following
#   attributes and methods:
#
#      is_completed
#         True if the player has completed the level.
#
#      begin_frame(self)
#         Called at the beginning of each frame whenever the
#         game is not paused.
#
#      reset(self)
#         Should return the level to its initial state.
#
##################################################################

import os, gc
from copy import deepcopy
from tempfile import mkstemp
from cPickle import Pickler, Unpickler
from albow.resource import resource_path
from albow.dialogs import ask
from albow.file_dialogs import look_for_file_or_directory

#------------------------------------------------------------------------

class HGame(object):
	#  levels_completed     [string]   List of level names
	#  level                custom     Level template currently loaded
	#  playing_level        custom     Level instance currently being played
	#  playing_level_dir    string     Dir containing levels being played
	#  level_name           string     Name of level being played or edited
	#  save_dir             string     Last save file dir
	#  save_name            string     Last save file name
	#  #level_set_dir        string     Dir containing custom level set
	#  #level_set_name       string     Name of custom level set
	#  unsaved_progress     boolean    Game progress needs saving
  #  testing              boolean    Test Level command in effect
	#  state                object     Any additional game state to be saved
	
	app_version = (1, 0, 0)  #  Version number of the game application
	app_magic = "????"       #  Magic for recognising files created by us
	
	#  Saved game file constants

	save_file_version = 1         #  Version number of data we write
	save_min_version = 1          #  Minimum data version we understand
	save_app_version = (1, 0, 0)  #  Earliest app version that understands what we write
	save_version_ranges = {}      #  For earlier data versions, version -> (min_app_version, max_app_version)
	save_file_magic = "SAVE"      #  Magic for recognising type of data
	
	#  Level file constants

	level_file_version = 1
	level_min_version = 1
	level_app_version = (1, 0, 0)
	level_version_ranges = {}
	level_file_magic = "LEVL"
	
	#  Filenames and  suffixes
	
	save_file_default_name = "SavedGame"
	
	save_file_suffix = ".gam"   #  Saved game files
	level_file_suffix = ".lev"  #  Level files
	level_set_suffix = ".lvs"   #  Level set directories
	
	#  Standard level directory, relative to Resources (without suffix)
	
	std_level_dir_name = "levels"

	# The following paths are relative to the directory containing
	# the Game directory or the .app bundle.
	
	default_save_dir_name = "Saves"
	default_custom_level_dir_name = "Mods"
	
	#  Optional feature flags
	
	disable_gc_during_play = False
	
	#  Default values of instance variables

	level = None
	playing_level = None
	playing_level_dir = None
	level_name = None
	save_dir = None
	save_name = None
	#level_set_dir = None
	#level_set_name = None
	unsaved_progress = False
	testing = False
	state = None

	def __init__(self):
		self.levels_completed = []

	def game_in_progress(self):
		"""Returns true if a level is loaded and a set of levels is being played
		(i.e. not just an isolated stand-alone level)."""
		return self.level_loaded() and self.playing_level_set()
	
	def level_loaded(self):
		"""Returns true if a level is currently loaded."""
		return self.level is not None
	
	def level_started(self):
		"""Returns true if a level has been started."""
		return self.playing_level is not None

	def level_in_progress(self):
		"""Returns true if a level is being played and it is not yet
		completed."""
		return self.level_started() and not self.level_is_completed()
	
	def level_is_completed(self):
		"""Returns true if the currently loaded level has been completed."""
		level = self.playing_level
		return level and level.is_completed
	
	def playing_level_set(self):
		"""Returns true if a level set is being played (whether a level is
		loaded or not)."""
		dir = self.playing_level_dir
		return dir and dir.endswith(self.level_set_suffix)
	
	def unload_level(self):
		"""Stop playing and remove any level currently loaded."""
		self.stop_level()
		self.playing_level = None
		self.level = None
		self.level_name = None
	
	def start_level(self):
		"""Start or restart the currently loaded level."""
		#print "HGame.start_level" ###
		level = deepcopy(self.level)
		self.playing_level = level
		level.reset()
		if self.disable_gc_during_play:
			#print "Disabling GC" ###
			gc.disable()
	
	def stop_level(self):
		"""Shut down any level currently being played."""
		#print "HGame.stop_level" ###
		if self.disable_gc_during_play:
			#print "Enabling GC" ###
			gc.enable()
		self.playing_level = None

	def new_game(self):
		"""Prepare for a new game by discarding the current level, clearing the
		list of completed levels and saved game file name."""
		self.stop_level()
		self.level = None
		del self.levels_completed[:]
		self.save_name = None
		self.state = self.new_state()
	
	def new_state(self):
		return None
	
	def load_game(self, path):
		"""Restores current level set and saved game progress from the specified
		file. If the level set can't be found, the user will be asked to look
		for it."""
		f = open(path, "rb")
		self.check_save_magic(f, self.app_magic)
		self.check_save_magic(f, self.save_file_magic)
		p = Unpickler(f)
		file_version = p.load()
		app_version = p.load()
		self.check_file_version(file_version, app_version,
			self.save_min_version, self.save_file_version,
			self.save_version_ranges)
		level_set_path = p.load()
		if not level_set_path:
			level_set_path = self.get_std_level_dir()
		levels_completed = p.load()
		state = p.load()
		f.close()
		if not os.path.exists(level_set_path):
			level_set_path = self.look_for_level_set(level_set_path)
			if not level_set_path:
				return
		self.stop_level()
		self.level = None
		self.levels_completed = levels_completed
		self.state = state
		self.set_playing_level_dir(level_set_path)
		self.set_save_path(path)
		self.unsaved_progress = False
	
	def look_for_level_set(self, orig_path):
		name = os.path.basename(orig_path)
		prompt = "Can't find '%s'. Would you like to look for it?" % name
		if ask(prompt, ["Yes", "No"]) == "Yes":
			return look_for_file_or_directory(name)

	def check_save_magic(self, f, expected):
		magic = f.read(len(expected))
		if magic <> expected:
			raise EnvironmentError("Does not seem to be a saved game file.")
	
	def check_file_version(self, file_version, app_version,
			min_version, cur_version, version_ranges):
		if not min_version <= file_version <= cur_version:
			if file_version > cur_version:
				mess = "Can only be opened by version %s.%s.%s or later." % app_version
			else:
				v1, v2 = version_ranges[file_version]
				vs1 = "%s.%s.%s" % v1
				vs2 = "%s.%s.%s" % v2
				mess = "Can only be opened by versions %s to %s." % (vs1, vs2)
			raise EnvironmentError(mess)
	
	def save_game(self, path):
		"""Saves current game progress in the specified file. If a non-standard
		level set is being played, the pathname of the level set is also
		recorded in the saved game file."""
		fd, tmp_path = mkstemp(dir = os.path.dirname(path))
		f = os.fdopen(fd, "wb")
		f.write(self.app_magic)
		f.write(self.save_file_magic)
		p = Pickler(f, 2)
		p.dump(self.save_file_version)
		p.dump(self.save_app_version)
		level_set_path = self.playing_level_dir
		if level_set_path == self.get_std_level_dir():
			level_set_path = ""
		p.dump(level_set_path)
		p.dump(self.levels_completed)
		p.dump(self.state)
		f.close()
		try:
			os.unlink(path)
		except EnvironmentError:
			pass
		os.rename(tmp_path, path)
		self.set_save_path(path)
		self.unsaved_progress = False
	
	def get_std_level_dir(self):
		"""Returns the full pathname of the directory containing the standard
		levels."""
		path = resource_path(self.std_level_dir_name) + self.level_set_suffix
		return os.path.abspath(path)
	
	def get_save_dir(self):
		"""Returns the full pathname of the directory last used to save a
		game, or the default save directory."""
		return self.save_dir or self.get_default_dir(self.default_save_dir_name)
	
	def get_save_path(self):
		return os.path.join(self.get_save_dir(),
			self.save_name or self.save_file_default_name)
	
	def get_playing_level_dir(self):
		"""Returns the full pathname of the last directory from which a
		level was played, or the default level directory."""
		path = self.playing_level_dir
		if not path:
			path = self.get_std_level_dir()
		return path
	
	def mark_current_level_completed(self):
		"""Add the current level to the list of completed levels for the
		current game."""
		if not self.testing and self.playing_level_set() and \
				not self.is_final_level():
			name = self.level_name
			if name not in self.levels_completed:
				self.levels_completed.append(name)
				self.unsaved_progress = True
	
	def is_final_level(self):
		"""If the current level is one that should not be progressed beyond
		even when it is completed, return true."""
		return False
	
	def load_next_uncompleted_level(self):
		path = self.next_uncompleted_level_path()
		if path:
			self.load_level(path)
		else:
			self.stop_level()
			self.level = None

	def next_uncompleted_level_path(self):
		"""Returns the full pathname of the next level file in the current
		level set that is not in the list of completed levels. If no such
		level is found, returns None."""
		ext = self.level_file_suffix
		dir = self.get_playing_level_dir()
		if os.path.isdir(dir):
			names = [name for name in os.listdir(dir) if name.endswith(ext)]
			names.sort()
			for name in names:
				if name not in self.levels_completed:
					return os.path.join(dir, name)
		return None
	
	def load_level(self, path):
		level = self.read_level(path)
		self.stop_level()
		self.level = level
		self.level_needs_saving = False
		dir, name = os.path.split(path)
		self.set_playing_level_dir(dir)
		self.level_name = name
	
	def set_playing_level_dir(self, dir):
		"""Sets the directory from which levels are being or to be played."""
		self.playing_level_dir = dir

	def begin_frame(self):
		"""Calls the begin_frame() method of the current level, if any, and
		checks whether the level has been completed. If so, the level is added
		to the list of completed levels."""
		level = self.playing_level
		if level:
			level.begin_frame()
			if level.is_completed:
				self.mark_current_level_completed()
	
	def set_save_path(self, path):
		self.save_dir, self.save_name = os.path.split(path)
	
	def get_custom_level_set_dir(self):
		"""If a custom level set is being played or edited, returns the full
		pathname of the directory containing that level set. Otherwise,
		returns the default custom levels directory."""
		dir = self.playing_level_dir
		if dir and dir.endswith(self.level_set_suffix) \
				and dir != self.get_std_level_dir():
			return os.path.dirname(dir)
		else:
			return self.get_default_custom_level_dir()
	
	def get_default_custom_level_dir(self):
		return self.get_default_dir(self.default_custom_level_dir_name)

	def get_default_dir(self, name):
		from albow.resource import resource_dir
		d = os.path.dirname
		base = d(d(resource_dir))
		return os.path.join(base, name)

	def level_number_and_title(self):
		"""Get level number and title from the current level filename.
		Expects the filename to be formatted as '<number>-<title>'. Underscores
		in the title are replaced by spaces. Returns a tuple (number, name).
		If there is no number, 0 is returned for the number."""
		number = 0
		title = ""
		name = self.level_name
		if name:
			name = os.path.splitext(name)[0].replace("_", " ")
			parts = name.split("-", 1)
			if len(parts) == 2:
				title = parts[1]
				try:
					number = int(parts[0])
				except ValueError:
					pass
			else:
				title = name
		return number, title
