################################################################
#
#   Humerus - HShell
#
#   A Shell subclasss designed to be used with an HGame.
#
################################################################

import os, sys
import pygame.display
from pygame.locals import FULLSCREEN, K_ESCAPE
from albow.shell import Shell
from albow.dialogs import alert, ask
from albow.file_dialogs import request_old_filename, request_new_filename
import options

class HShell(Shell):
	#  game      Game
	#  options   Options

	game_title = "Untitled Game"
	
	#  Screens - may be None if that screen type not used in game
	
	about_screen = None
	main_menu_screen = None
	play_screen = None
	
	has_level_editor = False

	def __init__(self, game, options):
		self.game = game
		self.options = options
		display = pygame.display.get_surface()
		Shell.__init__(self, display)
	
	def run(self):
		self.handle_command_line_options()
		Shell.run(self)
	
	def handle_command_line_options(self):
		if self.load_filename_argument():
			self.restart_level()
		elif self.options.new_game_option:
			self.new_game_cmd()
		else:
			self.default_startup()
	
	def default_startup(self):
		self.show_main_menu_screen()

	def load_filename_argument(self):
		filename = self.options.filename_argument
		if filename:
			path = os.path.abspath(filename)
			return self.load_file(path)
		else:
			return False
	
	def show_screen(self, screen):
		if screen:
			Shell.show_screen(self, screen)
	
	def show_about_screen(self):
		self.show_screen(self.about_screen)

	def show_main_menu_screen(self):
		self.show_screen(self.main_menu_screen)

	def show_play_screen(self, testing = False):
		if self.game.level_loaded():
			self.show_screen(self.play_screen)
		else:
			self.show_menu_screen()
	
	def load_file(self, path):
		"""Load a level set, level or saved game dpending on the filename suffix.
		Reports errors to the user. Returns true unless cancelled or an error
		occurs."""
		if path.endswith(self.game.level_set_suffix):
			return self.load_level_set(path)
		elif path.endswith(self.game.level_file_suffix):
			return self.load_level_file(path)
		elif path.endswith(self.game.save_file_suffix):
			return self.restore_game(path)
		return False
	
	def about_enabled(self):
		return self.about_screen is not None

	def about_cmd(self):
		"""Implementation of the 'About' command."""
		self.show_about_screen()
	
	def new_game_cmd(self):
		"""Implementation of the 'New Game' command."""
		if self.ask_save():
			game = self.game
			game.new_game()
			if self.load_next_uncompleted_level():
				if game.level_loaded():
					self.restart_level()
				else:
					self.no_levels_found()
					self.show_main_menu_screen()
	
	def no_levels_found(self):
		"""Called when a level set is found to contain no levels."""
		alert("No levels found.")
	
	def load_game_cmd(self):
		"""Implementation of the 'Load Game' command."""
		game = self.game
		if self.ask_save():
			dir = game.get_save_dir()
			path = request_old_filename(suffixes = [game.save_file_suffix],
				directory = dir)
			if path:
				self.restore_game(path)
	
	def restore_game(self, path):
		"""Restore a saved game from the specified path and start playing
		the next uncompleted level. Reports errors to the user. Returns true
		unless cancelled or an error occurs."""
		if self.load_game_file(path):
			if self.load_next_uncompleted_level():
				self.restart_level()
	
	def load_game_file(self, path):
		"""Load a saved game from the specified path. Reports errors to
		the user. Returns true unless cancelled or an error occurs."""
		try:
			self.game.load_game(path)
			return True
		except EnvironmentError, e:
			alert("Unable to load '%s': %s" % (
				os.path.basename(path), e))
		return False
	
	def save_game_enabled(self):
		"""Enable checker for the 'Save Game' command."""
		game = self.game
		return game.game_in_progress() and game.unsaved_progress

	def save_game_cmd(self):
		"""Implementation of the 'Save Game' command."""
		self.save_game(prompt = False)
	
	def save_game_as_enabled(self):
		"""Enable checker for the 'Save Game' command."""
		return self.game.game_in_progress()

	def save_game_as_cmd(self):
		"""Implementation of the 'Save Game As' command."""
		self.save_game(prompt = True)
	
	def save_game(self, prompt):
		"""Save the game. If prompt is true or no save pathname is set, ask
		for a pathname. Reports errors to the user. Returns true unless cancelled
		or an error occurs."""
		game = self.game
		old_path = game.get_save_path()
		if prompt or not game.save_name:
			new_path = request_new_filename(prompt = "Save game as:",
				suffix = game.save_file_suffix, pathname = old_path)
		else:
			new_path = old_path
		if new_path:
			try:
				game.save_game(new_path)
				return True
			except EnvironmentError, e:
				alert("Unable to save '%s': %s" % (
					os.path.basename(new_path), e))
		return False
	
	def resume_game_enabled(self):
		"""Enable checker for the 'Resume Game' command."""
		return self.game.level_in_progress()

	def resume_game_cmd(self):
		"""Implementation of the 'Resume Game' command."""
		self.resume_game()
	
	def resume_game(self):
		if self.game.level_loaded():
			self.show_play_screen()
	
	def restart_level_enabled(self):
		"""Enable checker for the 'Restart Level' command."""
		return self.game.level_loaded()
	
	def restart_level_cmd(self):
		"""Implementation of the 'Restart Level' command."""
		self.restart_level()
	
	def restart_level(self):
		#print "HShell.restart_level" ###
		if self.game.level_loaded():
			if self.game.level_in_progress():
				self.game.stop_level()
			self.game.start_level()
			self.show_play_screen()
	
	def load_next_uncompleted_level(self):
		"""Load the next uncompleted level, if any. Reports errors to the user.
		Returns false if an error occurred. If there are no more levels,
		sets game.level to None and returns true."""
		path = self.game.next_uncompleted_level_path()
		if path:
			return self.load_level_file(path)
		else:
			self.game.unload_level()
			return True

	def load_level_file(self, path):
		"""Load the specified level file. Reports errors to the user. Returns
		true unless cancelled or an error occurs."""
		try:
			self.game.load_level(path)
			return True
		except EnvironmentError, e:
			alert("Unable to load '%s': %s" % (
				os.path.basename(path), e))
		return False

	def open_level_set_cmd(self):
		"""Implementation of the 'Play Custom Levels' command."""
		if self.ask_save():
			path = request_old_filename(suffixes = [self.game.level_set_suffix],
				directory = self.game.get_custom_level_set_dir())
			if path:
				if self.load_level_set(path):
					self.restart_level()
	
	def load_level_set(self, path):
		"""Make the specified directory the current custom level set and load
		the first level. Reports errors to the user. Returns true unless cancelled
		or an error occurs."""
		game = self.game
		game.set_playing_level_dir(path)
		game.new_game()
		if self.load_next_uncompleted_level():
			if self.game.level_loaded():
				return True
			else:
				alert("No levels found.")
		return False
	
	def quit_cmd(self):
		self.quit()

	def ask_save(self):
		"""If there is unsaved game progress, ask the user whether to save it, and
		if the answer is yes, do so. Reports errors to the user. Returns true unless
		cancelled or an error occurred."""
		return self.ask_save_progress()
		
	def ask_save_progress(self):
		if self.game.unsaved_progress:
			resp = ask("Save game progress?", ["Yes", "No", "Cancel"])
			if resp == "Cancel":
				return False
			if resp == "Yes":
				if not self.save_game(prompt = True):
					return False
		return True

	def confirm_quit(self):
		"""Ask the user whether to really quit the game. If there is unsaved
		information, give an opportunity to do so. Returns true if quitting should
		go ahead."""
		return self.ask_quit() and self.ask_save()
	
	def ask_quit(self):
		return ask("Quit the game?") == "OK"

	def begin_frame(self):
		screen = self.current_screen
		if screen:
			screen.begin_frame()
			if self.screen_runs_game(screen):
				#print "HShell: calling game.begin_frame" ###
				game = self.game
				game.begin_frame()
				if game.level_is_completed() and not game.is_final_level():
					self.level_completed()
	
	def level_completed(self):
		if self.load_next_uncompleted_level() and self.game.level_loaded():
			self.restart_level()
		else:
			self.show_main_menu_screen()

	def screen_runs_game(self, screen):
		"""Returns true if the game should run while the given screen
		is the current screen."""
		return screen is self.play_screen

	def key_down(self, e):
		if e.key == K_ESCAPE:
			self.escape_key()
	
	def escape_key(self):
		if self.current_screen is self.main_menu_screen:
			self.resume_game()
		else:
			self.show_main_menu_screen()
