#---------------------------------------------------------------------------
#
#   HIIW - Items
#
#---------------------------------------------------------------------------

from albow.resource import get_image, get_sound
from albow.image_array import get_image_array

allow_destroyed_pages = True

#---------------------------------------------------------------------------

class Item(object):
	#  kind             string
	#  image            Surface
	#  inventory_rect   Rect      Position in inventory view
	
	pickup_sound = None
	inventory_rect = None
	
	def picked_up(self):
		sound = self.pickup_sound
		if sound:
			sound.play()
	
	def add_prerequisite_items(self, items):
		pass
	
	def use_up(self):
		return True

#---------------------------------------------------------------------------

class ConsumableItem(Item):
	#  units           int
	#
  #  initial_units   int
	
	def __init__(self):
		self.units = self.initial_units
	
	def use_up(self):
		self.units -= 1
		return self.units == 0
	
	def get_image(self):
		#print "ConsumableItem.get_image:", self, self.units ###
		return self.images[self.units - 1]
	
	image = property(get_image)

#---------------------------------------------------------------------------

class Quill(ConsumableItem):
	kind = 'quill'
	initial_units = 3
	images = get_image_array("quill.png", 3, border = 2)
	pickup_sound = get_sound("swish.ogg")

class Ink(ConsumableItem):
	kind = 'ink'
	initial_units = 5
	images = get_image_array("ink.png", 5, border = 2)
	pickup_sound = get_sound("gurgle.ogg")

class Paper(Item):
	kind = 'paper'
	image = get_image("page_blank.png")
	pickup_sound = get_sound("paper_rustle.ogg")

#---------------------------------------------------------------------------

class Page(Item):
	#  sample      Sample
	#  pre_sample  Sample
	#  destroyed   boolean
	#  enemy       Enemy
	
	kind = 'page'
	images = [get_image("page_written.png"), get_image("page_destroyed.png")]
	pickup_sound = get_sound("paper_rustle.ogg")
	is_dummy = False
	is_title = False
	pre_sample = None
	enemy = None
	
	def __init__(self, sample = None, destroyed = False):
		self.sample = sample
		self.destroyed = destroyed
		if sample:
			self.pre_sample = sample.pre_sample
	
	def add_prerequisite_items(self, items):
		if self.destroyed:
			items.add_unit(Quill)
			items.add_unit(Ink)
			#items.append(Quill())
			#items.append(Ink())
			items.append(Paper())

	def __repr__(self):
		smp = ""
		pre = ""
		sample = self.sample
		if self.sample:
			smp = "%s,act%s,scene%s,#%s" % (sample.play.name, sample.act_idx,
				sample.scene_idx, sample.speech_no)
		if self.pre_sample:
			pre = ",%s" % self.pre_sample.character
		return "Page<%s%s>" % (smp, pre)
	
	def get_image(self):
		return self.images[self.destroyed]
	
	image = property(get_image)
	
	def description(self):
		sample = self.sample
		if sample:
			return "%s-act%s-scene%s" % (sample.play.name, sample.act_idx, sample.scene_idx)
		else:
			return repr(self)

#---------------------------------------------------------------------------

class DummyPage(Page):
	is_dummy = True

class TitlePage(DummyPage):
	is_title = True
	
	def __init__(self, play):
		DummyPage.__init__(self)
		self.play = play

dummy_page = DummyPage()

#---------------------------------------------------------------------------

class Folio(Item):
	#  play               Play
	#  num_pages          int
	#  pages              [Page or DummyPage]
	#  missing_page_nos   [int]
	#  missing_pages      [Page]
	#  selected_page_no   int

	kind = 'folio'
	image = get_image("folio.png")
	pickup_sound = get_sound("book_rustle.ogg")
	selected_page_no = None
	min_folio_pages = 11
	
	def __init__(self, rnd, play):
		self.play = play
		num_missing_pages = len(play.samples)
		self.num_pages = max(self.min_folio_pages, 2 * num_missing_pages + 1)
		self.pages = [dummy_page] * self.num_pages
		self.pages[0] = TitlePage(play)
		seq = rnd.choose("missing", num_missing_pages, range(1, self.num_pages))
		self.missing_page_nos = list(seq)
		self.missing_page_nos.sort()
		for i in self.missing_page_nos:
			self.pages[i] = None
		self.missing_pages = []
		for i, sample in enumerate(play.samples):
			page_rnd = rnd.fork(i)
			if allow_destroyed_pages:
				destroyed = page_rnd.range("destroyed", 2)
			else:
				destroyed = False
			page = Page(sample, destroyed)
			self.missing_pages.append(page)
	
	def get_selected_page(self):
		n = self.selected_page_no
		if n is not None:
			return self.pages[n]
	
	def set_selected_page(self, page):
		if page:
			self.selected_page_no = self.pages.index(page)
		else:
			self.selected_page_no = None
	
	selected_page = property(get_selected_page, set_selected_page)
	
	def check_whether_completed(self):
#		print "Folio.check_whether_completed:", self ###
#		print "Correct pages are:",
#		for page in self.missing_pages:
#			print page.description(),
#		print
#		print "Current pages are:",
#		for page in self.pages:
#			if not page or page.is_dummy:
#				print "None",
#			else:
#				print page.description(),
#		print
		pages = self.pages
		for n, correct_page in zip(self.missing_page_nos, self.missing_pages):
			if pages[n] is not correct_page:
				#print "Page", n, "is wrong" ###
				return False
		return True

#---------------------------------------------------------------------------

item_kinds = [
	'quill', 'ink', 'paper', 'folio', 'page'
]
