#---------------------------------------------------------------------------
#
#   MWG - Playing screen
#
#---------------------------------------------------------------------------

import pygame as pg
from pygame import Rect, Color
from albow.widget import Widget
from albow.controls import Button
from albow.layout import Row, Column
from albow.resource import get_image
from albow import music
from main_commands import HIIWMainCommandScreen
from folio_view import FolioView
from game import game
from house import floor_size, HORZ, VERT, WEST, EAST, NORTH, SOUTH, UP, DOWN
from items import item_kinds
from room_styles import carpets
from options import options
from help import HelpButton, HelpBox

#---------------------------------------------------------------------------

floor_image = get_image("floor_wood.png")

wall_images = [
	get_image("wall_h_wood.png"),
	get_image("wall_v_wood.png"),
]

player_image = get_image("player.png")

stair_images = {
	(UP, WEST): get_image("stairs_west.png"),
	(UP, NORTH): get_image("stairs_north.png"),
	(DOWN, EAST): get_image("stairs_west.png"),
	(DOWN, SOUTH): get_image("stairs_north.png"),
}

#---------------------------------------------------------------------------

class FloorView(Widget):

	cell_size = 48
	margin = 12

	def __init__(self):
		m = 2 * self.margin
		s =  self.cell_size
		w = m + s * floor_size[0]
		h = m + s * floor_size[1]
		Widget.__init__(self, Rect((0, 0), (w, h)))
		self.add(HelpBox("play_screen.hlp", folio_not_visible))
	
	def draw(self, surf):
		floor = game.current_floor
		m = self.margin
		cs = self.cell_size
		for room in floor.rooms:
			self.draw_room(surf, room)
		self.draw_cells(surf, floor)
		self.draw_player(surf)
		self.draw_walls(surf, floor, HORZ)
		self.draw_walls(surf, floor, VERT)
	
	def draw_room(self, surf, room):
		r = self.bounds_to_rect(room.bounds)
		blit_tiled(surf, floor_image, r)
		n = room.carpet_no
		if n is not None:
			carpet = carpets[n]
			img = carpet.image
			ir = img.get_rect()
			ir.center = r.center
			surf.blit(img, ir)
	
	def draw_walls(self, surf, floor, dir):
		fw, fh = floor_size
		m = self.margin
		cs = self.cell_size
		walls = floor.walls[dir]
		wall_image = wall_images[dir]
		d = wall_image.get_size()[1-dir] // 2
		x = m - d
		for column in walls:
			y = m - d
			for wall in column:
				if wall.is_wall:
					surf.blit(wall_image, (x, y))
				if wall.obstacle:
					self.draw_obstacle(surf, wall.obstacle, dir, (x+d, y+d))
				y += cs
			x += cs
	
	def draw_obstacle(self, surf, obstacle, dir, p):
		image = obstacle.image
		r = image.get_rect()
		if dir == HORZ:
			r.midleft = p
		else:
			r.midtop = p
		surf.blit(image, r)
	
	def draw_player(self, surf):
		self.blit_in_cell(surf, player_image, game.state.player_location)

	def draw_cells(self, surf, floor):
		cx = 0
		for column in floor.cells:
			cy = 0
			for cell in column:
				self.draw_cell(surf, cell, (cx, cy))
				cy += 1
			cx += 1

	def draw_cell(self, surf, cell, coords):
		if cell.upstairs_dir is not None:
			self.draw_stairs(surf, cell, coords, UP, cell.upstairs_dir)
		elif cell.downstairs_dir is not None:
			self.draw_stairs(surf, cell, coords, DOWN, cell.downstairs_dir)
		furn = cell.furniture
		if furn:
			self.blit_in_cell(surf, furn.images[furn.facing], coords, 'topleft')
		item = cell.item
		if item:
			self.blit_in_cell(surf, item.image, coords)
	
	def draw_stairs(self, surf, cell, coords, up_down, dir):
		img = stair_images.get((up_down, dir))
		if img:
			self.blit_in_cell(surf, img, coords)
	
	def blit_in_cell(self, surf, image, coords, align = 'center'):
		rcell = self.cell_rect(coords)
		r = image.get_rect()
		setattr(r, align, getattr(rcell, align))
		surf.blit(image, r)

	def cell_rect(self, cell_coords):
		cx, cy = cell_coords
		m = self.margin
		cs = self.cell_size
		return Rect(m + cs * cx, m + cs * cy, cs, cs)
	
	def bounds_to_rect(self, bounds):
		m = self.margin
		cs = self.cell_size
		l, t, r, b = bounds
		x = m + cs * l
		y = m + cs * t
		w = cs * (r - l)
		h = cs * (b - t)
		return Rect(x, y, w, h)

#---------------------------------------------------------------------------

class ItemRenderer(object):

	selection_bg_color = (0x40, 0x80, 0xff) # Color("skyblue")
	selection_inflation = (4, 4)
	spacing = 5
	
	def __init__(self, surf, item_lists):
		self.surface = surf
		self.item_lists = item_lists

	def draw_items(self, kind, origin, hpitch = None, wrap = False, sel = None):
		items = self.item_lists[kind]
		if items:
			surf = self.surface
			left_margin, y = origin
			right_margin = surf.get_width() - left_margin
			w, h = items[0].image.get_size()
			s = self.spacing
			x = left_margin
			dx = hpitch or w + s
			dy = h + s
			for item in items:
				image = item.image
				rect = image.get_rect()
				rect.topleft = (x, y)
				item.inventory_rect = rect
				if item is sel:
					rsel = rect.inflate(self.selection_inflation)
					surf.fill(self.selection_bg_color, rsel)
				surf.blit(image, rect)
				x += dx
				if wrap and x + dx >= right_margin:
					x = left_margin
					y += dy

class InventoryView(Widget):

	bg_color = Color("bisque")
	item_h_spacing = 10
	item_v_spacing = 60
	margin = 10

	def __init__(self):
		Widget.__init__(self, Rect(0, 0, 200, 600))
	
	def draw(self, surf):
		r = ItemRenderer(surf, game.state.items)
		r.draw_items('quill', (10, 10), 30)
		r.draw_items('ink', (10, 50))
		r.draw_items('paper', (10, 80), 10)
		r.draw_items('folio', (10, 130), sel = game.state.selected_folio)
		r.draw_items('page', (10, 180), 35, wrap = True, sel = game.state.selected_page)
		
	def mouse_down(self, e):
		p = e.local
		state = game.state
		for item in state.iter_inventory_items():
			r = item.inventory_rect
			if r and r.collidepoint(p):
				state.selected_item = item
				kind = item.kind
				if kind == 'folio':
					state.selected_folio = item
				elif kind == 'page':
					state.selected_page = item
				return

#---------------------------------------------------------------------------

class HIIWPlayScreen(HIIWMainCommandScreen):

	music_playlist = music.get_playlist("playlist",
		random = not options.no_random_music_option)

	def __init__(self, shell):
		HIIWMainCommandScreen.__init__(self, shell)
		floor_view = FloorView()
		inventory_view = InventoryView()
		inventory_view.topleft = floor_view.topright
		folios_button = FolioButton(self, action = game.toggle_folio)
		help_button = HelpButton()
		buttons = Column([folios_button, help_button], margin = 10)
		inventory_view.add(buttons)
		buttons.bottom = inventory_view.height
		folio_view = FolioView(floor_view.rect)
		self.folio_view = folio_view
		self.add(floor_view)
		self.add(inventory_view)
		self.add(folio_view)
		self.hide_folio_view()

	def enter_screen(self):
		music.change_playlist(self.music_playlist)
		
	def key_down(self, e):
		k = e.key
		if k == pg.K_LEFT:
			game.move_player(-1, 0)
		elif k == pg.K_RIGHT:
			game.move_player(1, 0)
		elif k == pg.K_UP:
			game.move_player(0, -1)
		elif k == pg.K_DOWN:
			game.move_player(0, 1)
		else:
			HIIWMainCommandScreen.key_down(self, e)
	
	def show_folio_view(self):
		self.folio_view.visible = True
	
	def hide_folio_view(self):
		self.folio_view.visible = False
	
#	def toggle_folio_view(self):
#		self.folio_view.visible = not self.folio_view.visible

#---------------------------------------------------------------------------

class FolioButton(Button):
	
	#texts = ["Show Folio", "Hide Folio"]

	def __init__(self, shell, **kwds):
		self.shell = shell
		Button.__init__(self, "XXXX Page & Folio", **kwds)

	def get_visible(self):
		state = game.state
		return state.selected_folio or state.selected_page

	def get_text(self):
		state = game.state
		if self.shell.folio_view.visible:
			text = "Hide "
		else:
			text = "Show "
		things = []
		if state.selected_page:
			things.append("Page")
		if state.selected_folio:
			things.append("Folio")
		return text + " & ".join(things)
		
#---------------------------------------------------------------------------

def blit_tiled(surf, image, rect):
	iw, ih = image.get_size()
	r, b = rect.bottomright
	y = rect.top
	while y < b:
		h = min(ih, b - y)
		x = rect.left
		while x < r:
			w = min(iw, r - x)
			surf.blit(image, (x, y), (0, 0, w, h))
			x += iw
		y += ih

def folio_visible():
	return game.folio_visible

def folio_not_visible():
	return not folio_visible()
