#---------------------------------------------------------------------------
#
#   HIIW - Shell
#
#---------------------------------------------------------------------------

import os, sys
import pygame.display
from pygame.locals import *
from albow.root import Cancel
from albow.shell import Shell
from albow.dialogs import alert, ask, Dialog
from albow.file_dialogs import request_old_filename, request_new_filename
from albow.controls import Label, Button
from albow.layout import Row, Column, Grid
from albow.resource import get_image
from albow import music
from albow.music import change_music
from main_commands import HIIWMainCommandScreen
from intro_screen import IntroScreen
from credits_screen import CreditsScreen
import options
from play_screen import HIIWPlayScreen

#---------------------------------------------------------------------------

class HIIWShell(Shell):
	#  game      Game
	#  options   Options

	game_title = "Hark, I Impale\nWeasels!"
	has_level_editor = False
	
	#  Screens - may be None if that screen type not used in game
	
	about_screen = None
	main_menu_screen = None
	intro_screen = None
	play_screen = None
	
	def __init__(self, game, options):
		display = pygame.display.get_surface()
		Shell.__init__(self, display)
		self.game = game
		self.options = options
		self.main_menu_screen = HIIWMainMenuScreen(self, with_menu = True)
		self.intro_screen = IntroScreen(self)
		self.play_screen = HIIWPlayScreen(self)
		self.credits_screen = CreditsScreen(self)

	def run(self):
		self.handle_command_line_options()
		Shell.run(self)
	
	def handle_command_line_options(self):
		if self.load_filename_argument():
			self.resume_game()
		elif self.options.new_game_option:
			self.game.new_game()
			self.resume_game()
		else:
			self.default_startup()
	
	def default_startup(self):
		self.show_main_menu_screen()

	def load_filename_argument(self):
		"""If a filename was supplied on the command line, load it and
		return True. Otherwise, return False. Reports errors to the user."""
		filename = self.options.filename_argument
		if filename:
			path = os.path.abspath(filename)
			self.load_file(path)
			return True
		else:
			return False
	
	def show_screen(self, screen):
		if screen:
			Shell.show_screen(self, screen)
	
	def show_about_screen(self):
		self.show_screen(self.about_screen)

	def show_main_menu_screen(self):
		self.show_screen(self.main_menu_screen)
	
	def show_intro_screen(self):
		self.show_screen(self.intro_screen)

	def show_play_screen(self, testing = False):
		if self.game.game_in_progress():
			self.show_screen(self.play_screen)
		else:
			self.show_menu_screen()
	
	def show_credits_screen(self):
		self.show_screen(self.credits_screen)
	
	def load_file(self, path):
		"""Load a level set, level or saved game depending on the filename suffix.
		Reports errors to the user."""
		if path.endswith(self.game.save_file_suffix):
			self.restore_game(path)

	def about_enabled(self):
		return self.about_screen is not None

	def about_cmd(self):
		"""Implementation of the 'About' command."""
		self.show_about_screen()
	
	def new_game_cmd(self):
		"""Implementation of the 'New Game' command."""
		print "HIIWShell.new_game_cmd" ###
		self.ask_save()
		self.game.new_game()
		self.resume_game(intro = True)

	def load_game_cmd(self):
		"""Implementation of the 'Load Game' command."""
		game = self.game
		self.ask_save()
		dir = game.get_save_dir()
		path = request_old_filename(suffixes = [game.save_file_suffix],
			directory = dir)
		if not path:
			raise Cancel
		self.restore_game(path)

	def restore_game(self, path):
		"""Restore a saved game from the specified path. Reports errors to the user."""
		self.load_game_file(path)
		self.resume_game()
	
	def load_game_file(self, path):
		"""Load a saved game from the specified path. Reports errors to
		the user. Raises Cancel if cancelled or an error occurs."""
		try:
			self.game.load_game(path)
		except EnvironmentError, e:
			alert("Unable to load '%s': %s" % (
				os.path.basename(path), e))
			raise Cancel
	
	def save_game_enabled(self):
		"""Enable checker for the 'Save Game' command."""
		game = self.game
		return game.game_needs_saving()

	def save_game_cmd(self):
		"""Implementation of the 'Save Game' command."""
		self.save_game(prompt = False)
		alert("Game saved.")
	
	def save_game_as_enabled(self):
		"""Enable checker for the 'Save Game' command."""
		return self.game.game_in_progress()

	def save_game_as_cmd(self):
		"""Implementation of the 'Save Game As' command."""
		self.save_game(prompt = True)

	def save_game(self, prompt):
		"""Save the game. If prompt is true or no save pathname is set, ask
		for a pathname. Reports errors to the user. Raises Cancel if cancelled
		or an error occurs."""
		game = self.game
		old_path = game.get_save_path()
		if prompt or not game.save_name:
			new_path = request_new_filename(prompt = "Save game as:",
				suffix = game.save_file_suffix, pathname = old_path)
		else:
			new_path = old_path
		if not new_path:
			raise Cancel
		try:
			game.save_game(new_path)
		except EnvironmentError, e:
			alert("Unable to save '%s': %s" % (
				os.path.basename(new_path), e))
			raise Cancel

	def resume_game_enabled(self):
		"""Enable checker for the 'Resume Game' command."""
		return self.game.game_in_progress()

	def resume_game_cmd(self):
		"""Implementation of the 'Resume Game' command."""
		self.resume_game()
	
	def resume_game(self, intro = False):
		if self.game.game_in_progress():
			if intro and self.intro_screen:
				self.show_intro_screen()
			else:
				self.show_play_screen()
	
	def credits_cmd(self):
		self.show_credits_screen()
	
	def quit_cmd(self):
		self.quit()

	def ask_save(self):
		"""If there is anything to be saved, ask the user whether to save it, and
		if the answer is yes, do so. Reports errors to the user. Raises Cancel if
		cancelled or an error occurred."""
		self.ask_save_progress()
		
	def ask_save_progress(self):
		"""If there is unsaved game progress, ask the user whether to save it, and
		if the answer is yes, do so. Reports errors to the user. Raises Cancel if
		cancelled or an error occurred."""
		if self.game.game_needs_saving():
			resp = ask("Save game progress?", ["Yes", "No", "Cancel"])
			if resp == "Cancel":
				raise Cancel
			if resp == "Yes":
				self.save_game(prompt = True)

	def ask_quit(self):
		"""Make sure user really wants quit the game. Raises Cancel if cancelled
		or an error occurred."""
		if not ask("Exeunt game?") == "OK":
			raise Cancel

	def confirm_quit(self):
		"""Make sure user really wants quit the game. If there is unsaved
		information, give an opportunity to do so. Returns true if quitting should
		proceed. Raises Cancel if cancelled or an error occurred."""
		self.ask_quit()
		self.ask_save()
		return True
	
	def begin_frame(self):
		game = self.game
		screen = self.current_screen
		if screen:
			screen.begin_frame()
			if self.screen_runs_game(screen):
				game.begin_frame()
		if game.game_in_progress():
			if game.check_whether_completed():
				self.game_won()
	
	def game_won(self):
		alert("Excellent! All my words are now restored to their rightful arrangement.")
		#self.show_main_menu_screen()
		self.show_credits_screen()

	def screen_runs_game(self, screen):
		"""Returns true if the game should run while the given screen
		is the current screen."""
		return screen is self.play_screen

#	def key_down(self, e):
#		if e.key == K_ESCAPE:
#			self.escape_key()
#	
#	def escape_key(self):
#		if self.current_screen is self.main_menu_screen:
#			self.resume_game()
#		else:
#			self.show_main_menu_screen()
	
	def options_cmd(self):
		dlog = OptionsDialog(self)
		dlog.present()

#---------------------------------------------------------------------------

class HIIWMainMenuScreen(HIIWMainCommandScreen):

	title_image = get_image("title.jpeg")
	
	def add_title_and_menu(self, title, menu):
		title.topleft = (40, 40)
		menu.topleft = (title.left, title.bottom + 10)
		self.add(title)
		self.add(menu)

	def draw(self, surf):
		img = self.title_image
		r = img.get_rect()
		r.center = self.center
		surf.blit(img, r)

	def enter_screen(self):
		change_music(self.title_music)
	
	def return_key(self):
		self.shell.resume_game()

#---------------------------------------------------------------------------

class OptionsDialog(Dialog):

	def __init__(self, shell):
		self.shell = shell
		Dialog.__init__(self)
		emc = music.EnableMusicControl()
		mvc = music.MusicVolumeControl()
		controls = Grid([
			[Label("Enable Music"), emc],
			[Label("Music Volume"), mvc],
		])
		buttons = Row([
			Button("OK", self.ok),
			Button("Main Menu", self.main_menu),
		])
		contents = Column([controls, buttons], align = 'r', spacing = 20)
		contents.topleft = (20, 20)
		self.add(contents)
		self.shrink_wrap()

	def main_menu(self):
		self.shell.show_main_menu_screen()
		self.dismiss()
